"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.generate = generate;
exports.makeFile = makeFile;
exports.schemaToVuelidate = schemaToVuelidate;
exports.schemaItemToVuelidate = schemaItemToVuelidate;
const fs_1 = __importDefault(require("fs"));
const util_1 = __importDefault(require("util"));
const logger_1 = __importDefault(require("../utils/logger"));
const vexlidate_template_1 = require("../templates/vexlidate.template");
function generate(jsonSchema, outFile) {
    var _a;
    logger_1.default.process("vuelidate", (_a = jsonSchema === null || jsonSchema === void 0 ? void 0 : jsonSchema["x-documentConfig"]) === null || _a === void 0 ? void 0 : _a.documentName);
    const validatorsTemplateRaw = util_1.default.inspect(schemaToVuelidate(jsonSchema));
    const validatorsTemplate = validatorsTemplateRaw.replace(/'{{/g, "").replace(/}}'/g, "");
    const contentTemplate = `import * as validators from '@vuelidate/validators';
export default ${validatorsTemplate}`;
    logger_1.default.writing("vuelidate", outFile);
    return fs_1.default.writeFileSync(outFile, contentTemplate);
}
function makeFile(outDir) {
    const utilsFile = `${outDir}/utils.ts`;
    if (!fs_1.default.existsSync(utilsFile)) {
        fs_1.default.writeFileSync(utilsFile, (0, vexlidate_template_1.vexlidateTemplate)());
    }
    ;
    const indexFile = `${outDir}/index.ts`;
    if (!fs_1.default.existsSync(indexFile)) {
        fs_1.default.writeFileSync(indexFile, (0, vexlidate_template_1.indexTemplate)());
    }
    ;
}
function schemaToVuelidate(schema) {
    const validatorConfigs = {};
    const ObjectKeys = Object.keys(schema === null || schema === void 0 ? void 0 : schema.properties);
    ObjectKeys.forEach((key) => {
        const value = schema.properties[key];
        validatorConfigs[key] = schemaItemToVuelidate(schema, key, value);
    });
    return validatorConfigs;
}
function schemaItemToVuelidate(schema, key, value) {
    let validationConfig = {};
    if (schema.required && schema.required.includes(key)) {
        validationConfig.required = `{{validators.required}}`;
    }
    switch (value.type) {
        case 'string':
            if (value.format === 'email') {
                validationConfig.email = `{{validators.email}}`;
            }
            if (value.minLength) {
                validationConfig.minLength = `{{validators.minLength(${value.minLength})}}`;
            }
            if (value.maxLength) {
                validationConfig.maxLength = `{{validators.maxLength(${value.maxLength})}}`;
            }
            if (value.pattern === '^[a-zA-Z0-9]*$') {
                validationConfig.alphaNum = `{{validators.alphaNum}}`;
            }
            break;
        case 'number':
            validationConfig.numeric = `{{validators.numeric}}`;
            if (value.minValue && value.maxValue) {
                validationConfig.between = `{{validators.between(${value.minValue}, ${value.maxValue})}}`;
            }
            else if (value.minValue !== undefined) {
                validationConfig.minValue = `{{validators.minValue(${value.minValue})}}`;
            }
            else if (value.maxValue !== undefined) {
                validationConfig.maxValue = `{{validators.maxValue(${value.maxValue})}}`;
            }
            break;
        case 'object':
            const objectValidations = schemaToVuelidate(value);
            validationConfig = objectValidations;
            break;
        case 'array':
            const arrayValidations = schemaItemToVuelidate(schema, key, value.items);
            validationConfig = arrayValidations;
            break;
        case 'boolean':
            break;
        default:
            break;
    }
    if (value.format) {
        switch (value.format) {
            case 'email':
            case 'emailAddress':
                validationConfig.email = `{{validators.email}}`;
                break;
            case 'uri':
            case 'url':
                validationConfig.url = `{{validators.url}}`;
                break;
            case 'ipv4':
            case 'ipv6':
            case 'ipAddress':
                validationConfig.ipAddress = `{{validators.ipAddress}}`;
                break;
            case 'mac':
            case 'macAddress':
                validationConfig.macAddress = `{{validators.macAddress}}`;
                break;
            default:
                break;
        }
    }
    ;
    if (value.oneOf) {
        const oneOf = value.oneOf.map((item) => schemaItemToVuelidate(schema, key, item));
        validationConfig.oneOf = `{{validators.or(${oneOf})}}`;
    }
    return validationConfig;
}
;
exports.default = {
    generate,
    makeFile,
    schemaToVuelidate,
    schemaItemToVuelidate
};
